==> xdvik-22.77.1.tar.gz: xdvik-22.77.1/texk/xdvik/statusline.c <==
/*------------------------------------------------------------
  statusline for the xdvi(k) previewer

  written by S. Ulrich (ulrich@cis.uni-muenchen.de)  2000/02/25

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to
  deal in the Software without restriction, including without limitation the
  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
  sell copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
  IN NO EVENT SHALL PAUL VOJTA OR ANY OTHER AUTHOR OF THIS SOFTWARE BE
  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
  OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  ------------------------------------------------------------*/


#include "xdvi-config.h"
#include "xdvi.h"
#include "version.h"
#include "statusline.h"
#include "xm_menu.h"

#include "kpathsea/c-vararg.h"
#include "my-vsnprintf.h"

#include <X11/Xatom.h>
#include <X11/StringDefs.h>

# ifdef MOTIF
#  include <Xm/Label.h>
#  include <Xm/Frame.h>
#  include <Xm/TextF.h>
# else
#  include <X11/Xaw/Viewport.h>
#  include <X11/Xaw/Label.h>
# endif


Widget statusline;

static Boolean initialized = False;

/*
 * only print MAX_LEN characters to the statusline
 * (it's only 1 line after all)
 */
#define MAX_LEN 512

/* for saving the statusline string if the statusline is
 * destroyed and recreated
 */
static char g_string_savebuf[MAX_LEN + 2];

int global_statusline_h = 20;

#ifdef MOTIF
/*
 * This is a hack to block further processing of some events on widgets.
 */
static void block_event_callback(Widget w, XtPointer client_data,
				 XEvent * ev, Boolean * cont)
{
    UNUSED(w);
    UNUSED(client_data);
    UNUSED(ev);
    
    /* Don't propagate this event further down... */
    *cont = False;

    return;
}
#endif /* MOTIF */

/*
 * Create the statusline widget. To be called at the beginning
 * of the program, and when expert mode is switched off.
 *
 *  Side effects:
 *	sets <global_statusline_h> to the height of the statusline in pixels.
 */

Widget
create_statusline(
#ifdef MOTIF
		  Widget parent
#else
		  void
#endif		  
		  )
{
#ifndef MOTIF
    Position vport_h;
    Position clip_x;
    Position clip_w;
    static Position my_h = 0;
#endif

    /*
     * FIXME: is there a better way to set the y position depending on
     * the height of the widget?
     * It doesn't work to change the value of XtNy *after* creating
     * the widget!
     */

    if (!initialized) {
#ifndef MOTIF
	/*
	 * determine height of statusline (depending on the font used).
	 * This is not changeable at runtime, so it's determined once and
	 * for all at program start.
	 */
	statusline = XtVaCreateWidget("statusline", labelWidgetClass, vport_widget,
				      XtNlabel, (XtArgVal) "test",
				      NULL);
	XtVaGetValues(statusline, XtNheight, &my_h, NULL);
	global_statusline_h = my_h;
	XtDestroyWidget(statusline);
#endif
	initialized = True;
	/* initialize g_string_savebuf */
	sprintf(g_string_savebuf, "This is xdvik %s", XDVI_TERSE_VERSION_INFO);
    }
#ifndef MOTIF
    /* determine position and width of statusline */
    XtVaGetValues(clip_widget, XtNx, &clip_x, XtNwidth, &clip_w, NULL);
    XtVaGetValues(vport_widget, XtNheight, &vport_h, NULL);
    if (vport_h - my_h <= 0) {
	/* the return value is only here to justify the function declaration */
	return statusline;
    }
    statusline = XtVaCreateManagedWidget("statusline",
					 labelWidgetClass, vport_widget,
					 XtNlabel, (XtArgVal) g_string_savebuf,
					 XtNwidth, clip_w,
					 XtNx, clip_x - 1,	/* so that left border becomes invisible */
					 XtNy, vport_h - my_h,
					 XtNjustify, XtJustifyLeft,
					 /* same as for the buttons line */
					 XtNborder, (XtArgVal) resource.fore_Pixel,
					 NULL);
#else
    statusline = XtVaCreateManagedWidget("statusline",
					 xmTextFieldWidgetClass, parent,
					 XmNalignment, XmALIGNMENT_END,
					 XmNdepth, (XtArgVal) our_depth,
					 XmNvisual, (XtArgVal) our_visual,
					 XmNcolormap, (XtArgVal) our_colormap,
					 XmNbottomAttachment, XmATTACH_FORM,
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_FORM,
					 XmNleftOffset, 1,
					 XmNrightOffset, 1,
					 XmNbottomOffset, 1,
					 XmNtopOffset, 0,
					 XmNcursorPositionVisible, False,
					 XmNmarginWidth, 4,
					 XmNmarginHeight, 1,
					 XmNeditable, False,
					 XmNtraversalOn, False,
					 XmNvalue, g_string_savebuf,
					 NULL);

    /* Just block processing of interactive events on this widget, since we are
     * abusing this widget as kind of calculated size status display.
     * Admittedly this is a bit tricky... but unfortunately I don't know about
     * any better way to achieve the desired behaviour.
     */

    XtInsertEventHandler(statusline,
			 KeyPressMask | KeyReleaseMask |
			 ButtonPressMask | ButtonReleaseMask |
			 PointerMotionMask| PointerMotionHintMask |
			 ButtonMotionMask | FocusChangeMask,
			 True, block_event_callback,
			 (XtPointer)0, 0);

#endif

    return statusline;
}


void
toggle_statusline(void)
{
#ifdef MOTIF
    if ((expert_flag & XPRT_SHOW_STATUSLINE) == 0)
	XtUnmanageChild(statusline);
    else
	XtManageChild(statusline);
    //FIXME
/*     set_show_statusline_option(); */
#else
    static Boolean initialized = False;
    static Boolean statusline_mapped = False;

    Boolean make_statusline_visible = False;
    Boolean make_statusline_invisible = False;
    
    if (!initialized) {
	statusline_mapped = (expert_flag & XPRT_SHOW_STATUSLINE) != 0;
	initialized = True;
    }

    if ((expert_flag & XPRT_SHOW_STATUSLINE) == 0) {
	if (statusline_mapped)
	    make_statusline_invisible = True;
    }
    else {
	if (!statusline_mapped)
	    make_statusline_visible = True;
    }

    if (make_statusline_invisible) {
	XtDestroyWidget(statusline);
	statusline_mapped = False;
    }
    if (make_statusline_visible) {
	XtVaSetValues(vport_widget, XtNresizable, (XtArgVal)True, NULL);
	TRACE_GUI((stderr, "statusline: w %d, h %d", window_w, window_h));
	XtVaSetValues(vport_widget, XtNwidth, (XtArgVal)window_w, XtNheight, (XtArgVal)window_h, NULL);
	TRACE_GUI((stderr, "after statusline"));
	create_statusline();
	statusline_mapped = True;
    }
#endif /* MOTIF */
}


/*------------------------------------------------------------
 *  handle_statusline_resize
 *
 *  Arguments:
 *	void
 *
 *  Returns:
 *	void
 *
 *  Purpose:
 *	Resize the statusline when the total window size changes.
 *
 *------------------------------------------------------------*/

void
handle_statusline_resize(void)
{
#ifndef MOTIF
    if ((expert_flag & XPRT_SHOW_STATUSLINE) == 0) {
	return;
    }

    if (!statusline)
	return;

    /* apparently the x,y values of a widget can only be set at creation time, so
     * the following won't work:
     */

#if 0
    // BROKEN  Position vport_h, clip_x, clip_w;
    // BROKEN  static Position my_h = 0;
    // BROKEN
    // BROKEN  XtVaGetValues(clip_widget,
    // BROKEN                XtNx, &clip_x,
    // BROKEN                XtNwidth, &clip_w,
    // BROKEN                NULL);
    // BROKEN  XtVaGetValues(vport_widget,
    // BROKEN                XtNheight, &vport_h,
    // BROKEN                NULL);
    // BROKEN
    // BROKEN  XtUnmanageChild(statusline);
    // BROKEN  XtVaSetValues(statusline,
    // BROKEN                //             XtNlabel, (XtArgVal) "",
    // BROKEN                XtNwidth, clip_w,
    // BROKEN                XtNx, clip_x - 1,
    // BROKEN                XtNy, vport_h - my_h,
    // BROKEN                XtNborderWidth, 1,
    // BROKEN                XtNjustify, XtJustifyLeft,
    // BROKEN                XtNborder, (XtArgVal) resource.fore_Pixel,
    // BROKEN                NULL);
    // BROKEN  XtManageChild(statusline);
    // BROKEN  XFlush(DISP);
#endif

    /* only this will: */
    XtDestroyWidget(statusline);
    create_statusline();
#endif
}




/*
 * clear statusline by printing an empty message to it.
 */


static void
clear_statusline(void)
{
    if ((expert_flag & XPRT_SHOW_STATUSLINE) != 0) {
# ifdef MOTIF
	XmTextFieldSetString(statusline, " ");
# else
	XtVaSetValues(statusline, XtNlabel, " ", NULL);
# endif
	XFlush(DISP);
    }
    strcpy(g_string_savebuf, " ");
}


/* force a statusline update, no matter how busy the application is.
   Use this with care (only for important messages).
*/
void
force_statusline_update(void)
{
#ifdef MOTIF
    XmUpdateDisplay(top_level);
#else
    XEvent event;
    XSync(DISP, 0);
    while (XCheckMaskEvent(DISP, ExposureMask, &event))
        XtDispatchEvent(&event);
#endif /* MOTIF */
}


/*
 * timeout - if > 0, timeout in seconds after which the message will
 *	     be deleted again. If < 0, message will remain (until
 *	     another message overprints it)
 * fmt     - message, a C format string
 *
 * If expert mode is off, print <fmt> to the statusline; else, print
 * <fmt> to stdout, unless the `hushstdout' option is specified.
 */
static XtIntervalId clear_timeout_id = 0;

static void
clear_statusline_timer_proc(XtPointer client_data, XtIntervalId *id)
{
    UNUSED(client_data);
    UNUSED(id);
    
    clear_statusline();
    clear_timeout_id = 0;
}

void
print_statusline(status_timer_t timeout, _Xconst char *fmt, ...)
{
    va_list argp;
    char buf[MAX_LEN + 1];

    va_start(argp, fmt);

    if (!XtIsRealized(top_level) || !initialized || (expert_flag & XPRT_SHOW_STATUSLINE) == 0) {
	if (!resource.hush_stdout && strlen(fmt) > 0) {
	    fprintf(stdout, "xdvi: ");
	    (void)vfprintf(stdout, fmt, argp);
	    fprintf(stdout, "\n");
	    fflush(stdout);
	}
    }
    else {
	VSNPRINTF(buf, MAX_LEN, fmt, argp);	/* just trow away strings longer than MAX_LEN */
	/*
	 * save current contents of statusline so that toggling the statusline
	 * on and off will display the same text again
	 */
	strcpy(g_string_savebuf, buf);
#ifdef MOTIF
	XmTextFieldSetString(statusline, buf);
#else
	XtVaSetValues(statusline, XtNlabel, buf, NULL);
#endif
	if (timeout > 0) {
	    timeout *= 1000;	/* convert to miliseconds */

	    if (clear_timeout_id) {
		if (debug & DBG_EVENT)
		    fprintf(stderr, "%s:%d: removing timeout %ld\n", __FILE__, __LINE__, clear_timeout_id);
		XtRemoveTimeOut(clear_timeout_id);
	    }
	    clear_timeout_id = XtAppAddTimeOut(app, timeout,
					       clear_statusline_timer_proc, (XtPointer) NULL);
	}
    }
    va_end(argp);
}

==> xdvik-22.77.2.tar.gz: xdvik-22.77.2/texk/xdvik/statusline.c <==
/*------------------------------------------------------------
  statusline for the xdvi(k) previewer

  written by S. Ulrich (ulrich@cis.uni-muenchen.de)  2000/02/25

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to
  deal in the Software without restriction, including without limitation the
  rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
  sell copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
  EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
  IN NO EVENT SHALL PAUL VOJTA OR ANY OTHER AUTHOR OF THIS SOFTWARE BE
  LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
  OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
  WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  ------------------------------------------------------------*/


#include "xdvi-config.h"
#include "xdvi.h"
#include "version.h"
#include "statusline.h"
#include "xm_menu.h"

#include "kpathsea/c-vararg.h"
#include "my-vsnprintf.h"

#include <X11/Xatom.h>
#include <X11/StringDefs.h>

# ifdef MOTIF
#  include <Xm/Label.h>
#  include <Xm/Frame.h>
#  include <Xm/TextF.h>
# else
#  include <X11/Xaw/Viewport.h>
#  include <X11/Xaw/Label.h>
# endif


Widget statusline;

static Boolean initialized = False;

/*
 * only print MAX_LEN characters to the statusline
 * (it's only 1 line after all)
 */
#define MAX_LEN 512

/* for saving the statusline string if the statusline is
 * destroyed and recreated
 */
static char g_string_savebuf[MAX_LEN + 2];

int global_statusline_h = 20;

#ifdef MOTIF
/*
 * This is a hack to block further processing of some events on widgets.
 */
static void block_event_callback(Widget w, XtPointer client_data,
				 XEvent * ev, Boolean * cont)
{
    UNUSED(w);
    UNUSED(client_data);
    UNUSED(ev);
    
    /* Don't propagate this event further down... */
    *cont = False;

    return;
}
#endif /* MOTIF */

/*
 * Create the statusline widget. To be called at the beginning
 * of the program, and when expert mode is switched off.
 *
 *  Side effects:
 *	sets <global_statusline_h> to the height of the statusline in pixels.
 */

Widget
create_statusline(
#ifdef MOTIF
		  Widget parent
#else
		  void
#endif		  
		  )
{
#ifndef MOTIF
    Position vport_h;
    Position clip_x;
    Position clip_w;
    static Position my_h = 0;
#endif

    /*
     * FIXME: is there a better way to set the y position depending on
     * the height of the widget?
     * It doesn't work to change the value of XtNy *after* creating
     * the widget!
     */

    if (!initialized) {
#ifndef MOTIF
	/*
	 * determine height of statusline (depending on the font used).
	 * This is not changeable at runtime, so it's determined once and
	 * for all at program start.
	 */
	statusline = XtVaCreateWidget("statusline", labelWidgetClass, vport_widget,
				      XtNlabel, (XtArgVal) "test",
				      NULL);
	XtVaGetValues(statusline, XtNheight, &my_h, NULL);
	global_statusline_h = my_h;
	XtDestroyWidget(statusline);
#endif
	initialized = True;
	/* initialize g_string_savebuf */
	sprintf(g_string_savebuf, "This is xdvik %s", XDVI_TERSE_VERSION_INFO);
    }
#ifndef MOTIF
    /* determine position and width of statusline */
    XtVaGetValues(clip_widget, XtNx, &clip_x, XtNwidth, &clip_w, NULL);
    XtVaGetValues(vport_widget, XtNheight, &vport_h, NULL);
    if (vport_h - my_h <= 0) {
	/* the return value is only here to justify the function declaration */
	return statusline;
    }
    statusline = XtVaCreateManagedWidget("statusline",
					 labelWidgetClass, vport_widget,
					 XtNlabel, (XtArgVal) g_string_savebuf,
					 XtNwidth, clip_w,
					 XtNx, clip_x - 1,	/* so that left border becomes invisible */
					 XtNy, vport_h - my_h,
					 XtNjustify, XtJustifyLeft,
					 /* same as for the buttons line */
					 XtNborder, (XtArgVal) resource.fore_Pixel,
					 NULL);
#else
    statusline = XtVaCreateManagedWidget("statusline",
					 xmTextFieldWidgetClass, parent,
					 XmNalignment, XmALIGNMENT_END,
					 XmNdepth, (XtArgVal) our_depth,
					 XmNvisual, (XtArgVal) our_visual,
					 XmNcolormap, (XtArgVal) our_colormap,
					 XmNbottomAttachment, XmATTACH_FORM,
					 XmNleftAttachment, XmATTACH_FORM,
					 XmNrightAttachment, XmATTACH_FORM,
					 XmNleftOffset, 1,
					 XmNrightOffset, 1,
					 XmNbottomOffset, 1,
					 XmNtopOffset, 0,
					 XmNcursorPositionVisible, False,
					 XmNmarginWidth, 4,
					 XmNmarginHeight, 1,
					 XmNeditable, False,
					 XmNtraversalOn, False,
					 XmNvalue, g_string_savebuf,
					 NULL);

    /* Just block processing of interactive events on this widget, since we are
     * abusing this widget as kind of calculated size status display.
     * Admittedly this is a bit tricky... but unfortunately I don't know about
     * any better way to achieve the desired behaviour.
     */

    XtInsertEventHandler(statusline,
			 KeyPressMask | KeyReleaseMask |
			 ButtonPressMask | ButtonReleaseMask |
			 PointerMotionMask| PointerMotionHintMask |
			 ButtonMotionMask | FocusChangeMask,
			 True, block_event_callback,
			 (XtPointer)0, 0);

#endif

    return statusline;
}


void
toggle_statusline(void)
{
#ifdef MOTIF
    if ((expert_flag & XPRT_SHOW_STATUSLINE) == 0)
	XtUnmanageChild(statusline);
    else
	XtManageChild(statusline);

#if defined(NEW_MENU_CREATION) || defined(MOTIF)
    set_menu(&expert_flag, Act_set_expert_mode, check_expert_flag);
#else
    set_show_statusline_option();
#endif
#else
    static Boolean initialized = False;
    static Boolean statusline_mapped = False;

    Boolean make_statusline_visible = False;
    Boolean make_statusline_invisible = False;
    
    if (!initialized) {
	statusline_mapped = (expert_flag & XPRT_SHOW_STATUSLINE) != 0;
	initialized = True;
    }

    if ((expert_flag & XPRT_SHOW_STATUSLINE) == 0) {
	if (statusline_mapped)
	    make_statusline_invisible = True;
    }
    else {
	if (!statusline_mapped)
	    make_statusline_visible = True;
    }

    if (make_statusline_invisible) {
	XtDestroyWidget(statusline);
	statusline_mapped = False;
    }
    if (make_statusline_visible) {
	XtVaSetValues(vport_widget, XtNresizable, (XtArgVal)True, NULL);
	TRACE_GUI((stderr, "statusline: w %d, h %d", window_w, window_h));
	XtVaSetValues(vport_widget, XtNwidth, (XtArgVal)window_w, XtNheight, (XtArgVal)window_h, NULL);
	TRACE_GUI((stderr, "after statusline"));
	create_statusline();
	statusline_mapped = True;
    }
#endif /* MOTIF */
}


/*------------------------------------------------------------
 *  handle_statusline_resize
 *
 *  Arguments:
 *	void
 *
 *  Returns:
 *	void
 *
 *  Purpose:
 *	Resize the statusline when the total window size changes.
 *
 *------------------------------------------------------------*/

void
handle_statusline_resize(void)
{
#ifndef MOTIF
    if ((expert_flag & XPRT_SHOW_STATUSLINE) == 0) {
	return;
    }

    if (!statusline)
	return;

    /* apparently the x,y values of a widget can only be set at creation time, so
     * the following won't work:
     */

#if 0
    // BROKEN  Position vport_h, clip_x, clip_w;
    // BROKEN  static Position my_h = 0;
    // BROKEN
    // BROKEN  XtVaGetValues(clip_widget,
    // BROKEN                XtNx, &clip_x,
    // BROKEN                XtNwidth, &clip_w,
    // BROKEN                NULL);
    // BROKEN  XtVaGetValues(vport_widget,
    // BROKEN                XtNheight, &vport_h,
    // BROKEN                NULL);
    // BROKEN
    // BROKEN  XtUnmanageChild(statusline);
    // BROKEN  XtVaSetValues(statusline,
    // BROKEN                //             XtNlabel, (XtArgVal) "",
    // BROKEN                XtNwidth, clip_w,
    // BROKEN                XtNx, clip_x - 1,
    // BROKEN                XtNy, vport_h - my_h,
    // BROKEN                XtNborderWidth, 1,
    // BROKEN                XtNjustify, XtJustifyLeft,
    // BROKEN                XtNborder, (XtArgVal) resource.fore_Pixel,
    // BROKEN                NULL);
    // BROKEN  XtManageChild(statusline);
    // BROKEN  XFlush(DISP);
#endif

    /* only this will: */
    XtDestroyWidget(statusline);
    create_statusline();
#endif
}




/*
 * clear statusline by printing an empty message to it.
 */


static void
clear_statusline(void)
{
    if ((expert_flag & XPRT_SHOW_STATUSLINE) != 0) {
# ifdef MOTIF
	XmTextFieldSetString(statusline, " ");
# else
	XtVaSetValues(statusline, XtNlabel, " ", NULL);
# endif
	XFlush(DISP);
    }
    strcpy(g_string_savebuf, " ");
}


/* force a statusline update, no matter how busy the application is.
   Use this with care (only for important messages).
*/
void
force_statusline_update(void)
{
#ifdef MOTIF
    XmUpdateDisplay(top_level);
#else
    XEvent event;
    XSync(DISP, 0);
    while (XCheckMaskEvent(DISP, ExposureMask, &event))
        XtDispatchEvent(&event);
#endif /* MOTIF */
}


/*
 * timeout - if > 0, timeout in seconds after which the message will
 *	     be deleted again. If < 0, message will remain (until
 *	     another message overprints it)
 * fmt     - message, a C format string
 *
 * If expert mode is off, print <fmt> to the statusline; else, print
 * <fmt> to stdout, unless the `hushstdout' option is specified.
 */
static XtIntervalId clear_timeout_id = 0;

static void
clear_statusline_timer_proc(XtPointer client_data, XtIntervalId *id)
{
    UNUSED(client_data);
    UNUSED(id);
    
    clear_statusline();
    clear_timeout_id = 0;
}

static void
internal_print_statusline(status_timer_t timeout, _Xconst char *old_content, _Xconst char *fmt, va_list argp)
{
    if (!XtIsRealized(top_level) || !initialized || (expert_flag & XPRT_SHOW_STATUSLINE) == 0) {
	if (!resource.hush_stdout && strlen(fmt) > 0) { /* check for strlen since sometimes we clear the statusline
							   by printing "" to it, and we don't want that on stdout */
	    fprintf(stdout, "xdvi: ");
	    if (old_content != NULL)
		(void)fputs(old_content, stdout);
	    (void)vfprintf(stdout, fmt, argp);
	    fputc('\n', stdout);
	    fflush(stdout);
	}
    }
    else {
	char buf[MAX_LEN + 1];
	size_t offset = 0;
	
	if (old_content != NULL) {
	    offset += strlen(old_content);
	    strncpy(buf, old_content, MAX_LEN);
	    buf[MAX_LEN - 1] = '\0';
	}
	VSNPRINTF(buf + offset, MAX_LEN, fmt, argp);	/* just trow away strings longer than MAX_LEN */
	/*
	 * save current contents of statusline so that toggling the statusline
	 * on and off will display the same text again
	 */
	strcpy(g_string_savebuf, buf);
#ifdef MOTIF
	XmTextFieldSetString(statusline, buf);
#else
	XtVaSetValues(statusline, XtNlabel, buf, NULL);
#endif
	if (timeout > 0) {
	    timeout *= 1000;	/* convert to miliseconds */

	    if (clear_timeout_id) {
		if (debug & DBG_EVENT)
		    fprintf(stderr, "%s:%d: removing timeout %ld\n", __FILE__, __LINE__, clear_timeout_id);
		XtRemoveTimeOut(clear_timeout_id);
	    }
	    clear_timeout_id = XtAppAddTimeOut(app, timeout,
					       clear_statusline_timer_proc, (XtPointer) NULL);
	}
    }
}

/*
 * Append the varargs-string `fmt' to the currnent contents of the statusline
 * erasing it after `timeout' seconds if timeout > 0.
 */
void
statusline_append(status_timer_t timeout, _Xconst char *fmt, ...)
{
    _Xconst char *buf = NULL;
    va_list argp;
    
#ifdef MOTIF
    XtVaGetValues(statusline, XmNvalue, &buf, NULL);
#else
    XtVaGetValues(statusline, XtNlabel, &buf, NULL);
#endif

    va_start(argp, fmt);
    internal_print_statusline(timeout, buf, fmt, argp);
    va_end(argp);
}

/*
 * Print the varargs-string `fmt' to the currnent contents of the statusline,
 * erasing it after `timeout' seconds if timeout > 0.
 */

void
statusline_print(status_timer_t timeout, _Xconst char *fmt, ...)
{
    va_list argp;
    va_start(argp, fmt);
    internal_print_statusline(timeout, NULL, fmt, argp);
    va_end(argp);
}
